"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const assert = require("./assertions");
const sym = require("./symbols");
const common_1 = require("./common");
const errors_1 = require("./errors");
const validators_1 = require("./validators");
const assertions_1 = require("./assertions");
const chalk_1 = require("chalk");
/**
 * Validates that the Null, Object or Array value matches the schema.
 * @param schema	Null, Object or Array schema to match.
 * @param value		Value to be validated.
 * @param property	Name of the property being validated.
 */
function ValidateObject(schema, value, property) {
    // Null Validation
    if (schema === null) {
        Null(schema, value, property);
        return value;
    }
    // Array Validation
    if (Array.isArray(schema)) {
        // Ensure that the value is an array.
        try {
            assert.isArray(value);
        }
        catch (error) {
            throw new errors_1.ValidatorError(Array.name, property, value, error);
        }
        const array_error = new errors_1.MultipleValidationError(value);
        // Validate each element of the array.
        for (let element_idx = 0; element_idx < value.length; element_idx++) {
            const sub_property = `${property}[${element_idx}]`;
            try {
                // If multiple schemas use any validator
                if (schema.length > 1) {
                    let new_schema = validators_1.Any(schema);
                    value[element_idx] = ValidateRecursive(new_schema, value[element_idx], sub_property);
                }
                else {
                    value[element_idx] = ValidateRecursive(schema[0], value[element_idx], sub_property);
                }
            }
            catch (error) {
                array_error.child_errors.push(error);
            }
        }
        if (array_error.child_errors.length > 0) {
            throw array_error;
        }
        // Object Validation
    }
    else {
        // Ensure that the value is an object.
        try {
            assert.isObject(value);
            assert.isNull(value, assertions_1.INVERT);
            assert.isArray(value, assertions_1.INVERT);
        }
        catch (error) {
            throw new errors_1.ValidatorError(Object.name, property, value, error);
        }
        const object_error = new errors_1.MultipleValidationError(value);
        // Validate the fields of the object.
        for (const field of Object.getOwnPropertyNames(schema)) {
            try {
                value[field] = ValidateRecursive(schema[field], value[field], `${property}.${field}`);
            }
            catch (error) {
                object_error.child_errors.push(error);
            }
        }
        if (object_error.child_errors.length > 0) {
            throw object_error;
        }
    }
    return value;
}
exports.ValidateObject = ValidateObject;
;
function ValidateFunction(schema, value, property) {
    // Type Validation
    if (schema[sym.Validator] === sym.TypeValidator) {
        let type_name = schema[sym.TypeValidator];
        try {
            assert.isSameTypeName(type_name, common_1.objectType(value));
        }
        catch (error) {
            throw new errors_1.ValidatorError(type_name, property, value, error);
        }
    }
    else if (schema[sym.Validator] === sym.OptionsValidator) {
        const schemas = schema[sym.OptionsValidator].schemas;
        const option = schema[sym.OptionsValidator].option;
        const option_name = schema[sym.Metadata].name;
        const match_error = new errors_1.NotMatchAnyError(value);
        // Check each schema if (ALL); Check first valid schema (ANY).
        for (let schema_idx = 0; schema_idx < schemas.length; schema_idx++) {
            try {
                value = ValidateRecursive(schemas[schema_idx], value, property);
                if (option === validators_1.ValidationOptions.any) {
                    match_error.child_errors = [];
                    break;
                }
            }
            catch (error) {
                match_error.child_errors[schema_idx] = error;
            }
        }
        if (match_error.child_errors.length > 0) {
            throw new errors_1.ValidatorError(option_name, property, value, match_error);
        }
    }
    else if (schema[sym.Validator] === sym.CustomValidator) {
        let validator_name = schema[sym.Metadata].name;
        try {
            value = schema[sym.CustomValidator](value, property);
        }
        catch (error) {
            throw new errors_1.ValidatorError(validator_name, property, value, error);
        }
    }
    return value;
}
exports.ValidateFunction = ValidateFunction;
;
/**
 * Validates that the value is string literal.
 * @param schema    String literal to match.
 * @param value     Value to be validated.
 * @param property  Name of the property being validated.
 */
function LiteralString(schema, value, property) {
    try {
        assert.isString(value);
        assert.isEqual(schema, value);
    }
    catch (error) {
        throw new errors_1.ValidatorError(LiteralString.name, property, value, error);
    }
    return value;
}
exports.LiteralString = LiteralString;
;
/**
 * Validates that the value is number literal.
 * @param schema    Number literal to match.
 * @param value     Value to be validated.
 * @param property  Name of the property being validated.
 */
function LiteralNumber(schema, value, property) {
    try {
        assert.isNumber(value);
        assert.isEqual(schema, value);
    }
    catch (error) {
        throw new errors_1.ValidatorError(LiteralNumber.name, property, value, error);
    }
    return value;
}
exports.LiteralNumber = LiteralNumber;
;
/**
 * Validates that the value is boolean literal.
 * @param schema    Boolean literal to match.
 * @param value     Value to be validated.
 * @param property  Name of the property being validated.
 */
function LiteralBoolean(schema, value, property) {
    try {
        assert.isBoolean(value);
        assert.isEqual(schema, value);
    }
    catch (error) {
        throw new errors_1.ValidatorError(LiteralBoolean.name, property, value, error);
    }
    return value;
}
exports.LiteralBoolean = LiteralBoolean;
;
/**
 * Validates that the value is Symbol.
 * @param schema    Symbol schema to match.
 * @param value     Value to be validated.
 * @param property  Name of the property being validated.
 */
function LiteralSymbol(schema, value, property) {
    try {
        assert.isSymbol(value);
        assert.isEqual(schema, value);
    }
    catch (error) {
        throw new errors_1.ValidatorError(LiteralSymbol.name, property, value, error);
    }
    return value;
}
exports.LiteralSymbol = LiteralSymbol;
;
/**
 * Validates that the value is undefined.
 * @param schema    Redundant.
 * @param value     Value to be validated.
 * @param property  Name of the property being validated.
 */
function Undefined(schema, value, property) {
    try {
        assert.isUndefined(value);
    }
    catch (error) {
        throw new errors_1.ValidatorError(Undefined.name, property, value, error);
    }
    return value;
}
exports.Undefined = Undefined;
;
/**
 * Validates that the value is null.
 * @param schema    Redundant.
 * @param value     Value to be validated.
 * @param property  Name of the property being validated.
 */
function Null(schema, value, property) {
    try {
        assert.isNull(value);
    }
    catch (error) {
        throw new errors_1.ValidatorError(Null.name, property, value, error);
    }
    return value;
}
exports.Null = Null;
;
/**
 * Throws an unknown type validation error.
 * @param schema    Redundant.
 * @param value     Value to be validated.
 * @param property  Name of the property being validated.
 */
function Unknown(schema, value, property) {
    throw new errors_1.ValidatorError(Undefined.name, property, value, new Error('Unable to validate unknown type.'));
}
exports.Unknown = Unknown;
;
function ValidateRecursive(schema, value, property) {
    if (typeof schema === 'object') {
        value = ValidateObject(schema, value, property);
    }
    else if (typeof schema === 'function') {
        value = ValidateFunction(schema, value, property);
    }
    else if (typeof schema === 'string') {
        value = LiteralString(schema, value, property);
    }
    else if (typeof schema === 'number') {
        value = LiteralNumber(schema, value, property);
    }
    else if (typeof schema === 'boolean') {
        value = LiteralBoolean(schema, value, property);
    }
    else if (typeof schema === 'symbol') {
        value = LiteralSymbol(schema, value, property);
    }
    else if (typeof schema === 'undefined') {
        value = Undefined(schema, value, property);
    }
    else {
        value = Unknown(schema, value, property);
    }
    return value;
}
function banner(log) {
    log(`\n${chalk_1.default.bgRedBright(' '.repeat(31))}\n${chalk_1.default.bgRedBright('  ')}${chalk_1.default.bold(' Validate TypeScript Error ')}${chalk_1.default.bgRedBright('  ')}\n${chalk_1.default.bgRedBright(' '.repeat(31))}\n`);
}
exports.banner = banner;
function validate(schema, value, log = console.log, name = '') {
    try {
        ValidateRecursive(schema, value, name);
    }
    catch (error) {
        if (error instanceof errors_1.ValidationError) {
            if (!common_1.isUndefined(log)) {
                // banner(log);
                log('');
                log(error.trace);
                log('');
            }
            throw new Error(error.message);
        }
        else {
            throw error;
        }
    }
    return value;
}
exports.validate = validate;
//# sourceMappingURL=validate.js.map